/*
 *	Copyright (c) 1993-1996 by Research Computer Center (Expert Team)
 *	Moscow State University, Moscow, Russia
 *	All rights reserved.
 *
 *	%W%     %E%     (c) SRCC
 */

           /***====================================***/
           /***     C++  FRONT  END  COMPILER      ***/
           /***====================================***/


         /**********************************************/
         /**********************************************/
         /***                                        ***/
         /***----------------------------------------***/
         /***            Module FE-LIMIT.H           ***/
         /***----------------------------------------***/
         /***                                        ***/
         /***                                        ***/
         /***      I M P L E M E N T A T I O N       ***/
         /***                                        ***/
         /***               L I M I T S              ***/
         /***                                        ***/
         /***                                        ***/
         /***        Based on Third Proposal         ***/
         /***               (May 1993)               ***/
         /***                                        ***/
         /***                                        ***/
         /**********************************************/
         /**********************************************/


/***************************************************************
 *                                                             *
 *  SOURCE:                                                    *
 *                                                             *
 *  Paul Stone. "C++ Implementation Limits. Third Proposal".   *
 *  ANSI DOC NO: X3J16/930070                                  *
 *                                                             *
 ***************************************************************/


#ifndef _LIMIT_H
#define _LIMIT_H

                /* Nesting levels of compound statements, iteration  */
                /* and selection control structures, and conditional */
                /* inclusion                                         */

#define  lim_NEST_COMPOUNDS               256
#define  lim_NEST_ITERATIONS              256
#define  lim_NEST_SELECTIONS              256
#define  lim_NEST_CONDITIONAL_INCLUSION   256

                /* Pointer, array and function declarators (in any     */
                /* combinations) modifying an arithmetic, a structure, */
                /* a union, or an incomplete type in a declaration     */

#define  lim_DECL_PTR_ADR_FNC             256

                /* Nesting levels of parenthesised declarators         */
                /* (expressions) within a full declarator (expression) */

#define  lim_NEST_PAREN_DECL              256
#define  lim_NEST_PAREN_EXPR              256

                /* Significant initial characters in an internal       */
                /* identifier/macro name or in an external identifier  */

#define  lim_SIGNIFICANT_INTERNAL        1024
#define  lim_SIGNIFICANT_EXTERNAL        1024

                /* External identifiers in one translation unit */

#define  lim_EXTERNAL_IDENTIFIERS       65536

                /* Identifiers with block scope declared in one block */

#define  lim_BLOCK_IDENTIFIERS            512

                /* Macro identifiers simultaneously defined */
                /* in one translation unit                  */

#define  lim_MACRO_IDENTIFIERS          65536

                /* Parameters in one fuction definition and one */
                /* function call                                */

#define  lim_FUNCTION_PARAMETERS          256
#define  lim_FUNCTION_ARGUMENTS           256

                /* Parameters in one macro definition and one */
                /* macro invocation                           */

#define  lim_MACRO_PARAMETERS             256
#define  lim_MACRO_ARGUMENTS              256

                /* Characters in a logical source line and in a character */
                /* string/wide string literal (after concatenation)       */

#define  lim_LINE_LENGTH                 65536
#define  lim_LITERAL_LENGTH              65536

                /* Bytes in an object (in a hosted environment only) */

#define  lim_OBJECT_SIZE               262144

                /* Nesting levels for #included files */

#define  lim_NEST_INCLUDES                256

                /* Case label for a switch statement (excluding */
                /* those for any nested switch statements)      */

#define  lim_CASE_LABELS                  257

                /* Data members in a single enumeration */

#define  lim_STRUCT_MEMBERS             16384

                /* Enumeration constants in a single enumeration */

#define  lim_ENUM_CONSTANTS              4096

                /* Levels of nested structure or union definitions */
                /* in a single struct-declaration-list             */

#define  lim_NEST_STRUCTS                 256

                /* Functions registered by atexit(). See ISO C 9899, */
                /* 7.10.4.2. Note: this is a runtime, rather than    */
                /* translation, limit                                */

#define  lim_AT_EXIT_FUNCTIONS           2048


   /*-----------------   C++-specific limits   -------------------*/


                /* Direct and indirect base classes (count */
                /* of edges in the inheritance graph)      */

#define  lim_ALL_BASES                  16384

                /* Direct base classes per class */

#define  lim_DIRECT_BASE_CLASSES         1024

                /* Depth of nested class definitions, i.e.,      */
                /*                                               */
                /*     class S1 {                                */
                /*       class S2 {                              */
                /*         class S3 {                            */
                /*           ...                                 */
                /*         };                                    */
                /*       };                                      */
                /*     };                                        */
                /*                                               */
                /* Note: NEST_CLASSES may be redunant with       */
                /* NEST_STRUCTS.                                 */

#define  lim_NEST_CLASSES                 256

                /* Non-static base class members in a single */
                /* class object                              */

#define  lim_BASE_CLASS_MEMBERS          4096

                /* Non-static class members in a single class object.  */
                /* Includes the base class members for derived classes.*/
                /* Excludes compiler-generated members                 */

#define  lim_CLASS_MEMBERS              16384

                /* Abstract, type conversion ('operator T()')        */
                /* functions and overloaded constructorsin one class */

#define  lim_ABSTRACT_FUNCTIONS          4096
#define  lim_CONVERSION_FUNCTIONS        1024
#define  lim_OVERLOADED_CONSTRUCTORS     1024

                /* Overloaded functions for a given name */

#define  lim_OVERLOADED_FUNCTIONS        1024

                /* Virtual functions accessible in a given class, */
                /* including pure-virtuals not redefined          */

#define  lim_VIRTUAL_FUNCTIONS           1024

                /* Non-virtual functions accessible in a given class, */
                /* including automatically generated functions, but   */
                /* excluding constructors, destructors, operator      */
                /* new/delete, and static function members            */

#define  lim_NON_VIRTUAL_FUNCTIONS       1024

                /* Virtual base subobjects per class object */

#define  lim_VIRTUAL_BASE_SUBOBJECTS     1024

                /* Static members, friend declarations and  */
                /* access control declarations in one class */

#define  lim_STATIC_MEMBERS              1024
#define  lim_FRIENDS                     4096
#define  lim_ACCESS_DECLARATIONS         4096

                /* Initializations of base classes or members in */
                /* a constructor definition, e.g.,               */
                /*                                               */
                /*      T::T() : a(1), b(2), ... { }             */

#define  lim_MEM_INITIALIZERS           32768

                /* Scope qualifications of one identifier, e.g., */
                /*                                               */
                /*        BASE1 :: BASE2 :: BASE3 :: id          */

#define  lim_SCOPE_QUALIFIERS            4096

                /* 'extern "lang" { }' nesting levels */

#define  lim_NEST_EXTERNS                1024

                /* Template arguments in a template declaration */

#define  lim_TEMPLATE_ARGUMENTS           256

                /* Handlers per try block */

#define  lim_HANDLERS_PER_TRY_BLOCK       256

                /* Throw specifications on a single function         */
                /* declaration; that is, the number or type-id's     */
                /* in the type-id-list of an exception-specification */

#define  lim_EXCEPTION_SPECS              256

                /* (Added by Expert group)                             */
                /* the maximum number of using directives in one block */

#define  lim_USED_NAMESPACES              256


	/************************************************/
	/*             Preprocessor limits              */
	/************************************************/

#define MaxName        80
#define MaxPath        79

#endif

