/*
 *	Copyright (c) 1995-96 by ACE Associated Computer Experts bv,
 *	Amsterdam, The Netherlands
 *	All rights reserved.
 *
 *	%W%	%E%	(c) ACE
 */

#include <string.h>
#include <setjmp.h>
#include <assert.h>

#include <exception>

#ifdef DEBUG
#include <stdio.h>
#endif

#include "fe-limit.h"

_USING_STD(terminate)
_USING_STD(unexpected)
_USING_STD(bad_exception)
_USING_STD(uncaught_exception)


/*
 * Type: pointer to destructor function
 */
typedef void* (*DF) ( void*, int );

extern "C" void _AddType ( char* TypeCode, void* ObjectPart );
extern "C" void _AddDtorChain ( void** Link, DF Dtor );
extern "C" void _CutChain ( void );
extern "C" void _DestroyObjects ( void );
extern "C" void _DtorIterator ( void*, int, int, DF);
extern "C" int  _CompareTypes ( char* TypeCode );
extern "C" void _NullThrow ( DF Dtor );
extern "C" void *_RegistryObject (void *Object, DF Dtor );
extern "C" int  _PrepareException (void);
extern "C" void _UnwindFor (void*);
extern "C" void _ReThrow ();
extern "C" void _CheckUnexpected (char *);
extern "C" void *_AddFalseDelete (void *, void *, void *);

extern char _ThrowParam[];

/*
 *  The maximum SIZE of the type chain: the maximum
 *  amount of type modifier codes plus the size for
 *  the code of the "basic" type.
 */
#define  MAXTYPECHAIN  lim_DECL_PTR_ADR_FNC+1

/*
 * Direct and indirect base classes for the given class
 * (this value may be reduced for the cases of exceptions
 * because the original value of lim_ALL_BASES - 16384 -
 * seems to be too large)
 */
#define	MAXBASES	lim_ALL_BASES

typedef struct {
	char *TypeCode;
	void *Value;
} PAIR;

/* Structure itself */
static PAIR	_ThrowTypes [ 1+3+lim_ALL_BASES ];

/* Current amount of elements in _ThrowTypes */
int  _ThrowTypesCnt = 0;

void *_CurrentPart;
void *_ThrowBufferDtor;

bool _StillUncaught;

static bool _InsideUnexpected = false;

/*
 * Pointer to the last automatic object
 * with destructor - the head of the chain.
 */
void** _DtorChain = 0;

void
_AddType ( char* TypeCode, void* ObjectPart )
{
#ifdef DEBUG
	printf ("_AddType (%p(%s), %p)\n", TypeCode, TypeCode, ObjectPart);
#endif

	_ThrowTypes[_ThrowTypesCnt].TypeCode = TypeCode;
	_ThrowTypes[_ThrowTypesCnt].Value = ObjectPart;
	_ThrowTypesCnt++;
}

/*
 * void _AddDtorChain ( void**, DF )
 *      -------------
 *
 * Run-time function for adding new element to the
 * destructor chain.
 *
 *                to the previous
 *                     object
 *                       ^
 *           ...         |
 *       +-> Link      --+           -----------
 *       |   DtorAddr  ------------>|  ~A code  |
 *       |   Object of type A        -----------
 *       |   ...
 *       |
 *  _DtorChain
 *
 */
void
_AddDtorChain ( void** Link, DF Dtor )
{
	*Link = _DtorChain;	/* Storing previous pointer from _DtorChain */
	_DtorChain = Link;	/* Saving Link address in _DtorChain head   */

#ifdef DEBUG
	printf ("_AddDtorChain (%p, %p): prev = %p\n", Link, Dtor, *Link);
	printf ("*Link+2 =%p\n", Link+2);
#endif
    int tmp= reinterpret_cast<int>(Dtor);
	*(Link+1)= reinterpret_cast<void*>(tmp);	/* Storing destructor's address */
}

void *
_RegistryObject (void *Object, DF Dtor )
{
	void **ret = ((void**) Object)-2;

#ifdef DEBUG
	printf (
		"_RegistryObject (%p, %p): prev = %p\n",
		Object, Dtor, _DtorChain
	);
#endif

	*ret = _DtorChain;	/* Storing previous pointer from _DtorChain */
    int tmp= reinterpret_cast<int>(Dtor);
	*(ret+1) = (void*)tmp;	/* Saving Link address in _DtorChain head   */
	_DtorChain = ret;

	return Object;
}

static void *
CallDelete (void *p, int)
{
	DF func = *(DF*) ((void**)p+2);
	void *arg = *((void**)p+1);

	return func(arg, 0);
}

void *
_AddFalseDelete (void *res, void *temp, void *del)
{
	void **real_temp = (void**) temp;

	_RegistryObject (real_temp+2, CallDelete);
	real_temp[3] = res;
	real_temp[4] = del;
	return res;
}

/*
 *  void  _DestroyObjects ( void )
 *        ---------------
 *
 *  Run-time function for applying an appropriate destructor
 *  to every object in destructor chain.
 *
 *                          ^
 *              ...         |
 *         +--> Link     ---+     -----------
 *         |    DtorAddr ------->|  ~A code  |
 *         |    A a               -----------
 *         |    ...
 *         |
 *    _DtorChain
 *
 */

void
_DestroyObjects ( void )
{
	void**	OldDtorChain;

	void*	ObjAddr;   /* object address     */
	DF	DtorAddr;  /* destructor address */

	void **chain = _DtorChain;
	static bool r_calls;

	if (r_calls)
		return;
	else
		r_calls = true;

	while ( chain != 0 )  {

		OldDtorChain = chain;
		chain = (void**)(*chain);

		DtorAddr = (DF)(*(OldDtorChain+1));

#ifdef DEBUG
		printf ("Destroy Link = %p\n", OldDtorChain);
#endif

		if ( DtorAddr == 0 )  {
			/* try-block marker! */
			long *Buf = *(long**)(OldDtorChain+2);

#ifdef DEBUG
			printf ("LONGJMP = %p\n", Buf);
#endif
			_DtorChain = chain;
			r_calls = false;
			longjmp (Buf,1);
		} else  {
			/* ordinary chain element */
			ObjAddr = OldDtorChain+2;

#ifdef DEBUG
			printf ("%p (%p)\n", DtorAddr, ObjAddr);
#endif

			DtorAddr (ObjAddr,0);	/* calling to destructor */
		}
	}

	/* We have found the end of the chain, BUT */
	/* we haven't found the marker!- this is   */
	/* run-time error, hence we have to invoke */
	/* the standard terminate() function...    */

	r_calls = false;
	terminate();
}

void
_UnwindFor (void *stop)
{
	void**	OldDtorChain;

	void*	ObjAddr;   /* object address     */
	DF	DtorAddr;  /* destructor address */

#ifdef DEBUG
	printf ("UnwindFor: stop = %p; DtorChain = %p\n", stop, _DtorChain);
#endif

	if (_DtorChain == 0)
		return;

	while ((stop == 0 && _DtorChain != 0) ||
#ifndef BACKAUTO
			// corrected 02.02.98 ">= stop" changed to "> stop"
		(_DtorChain > stop && _DtorChain != 0)
#else
		_DtorChain < stop
#endif
	) {

		OldDtorChain = _DtorChain;
		_DtorChain = (void**)(*_DtorChain);

		DtorAddr = (DF)(*(OldDtorChain+1));

#ifdef DEBUG
		printf ("New Dtor Chain = %p\n", _DtorChain);
#endif

		if ( DtorAddr != 0 )  {
			/* ordinary chain element */
			ObjAddr = OldDtorChain+2;

#ifdef DEBUG
			printf ("%p (%p)\n", DtorAddr, ObjAddr);
#endif

			DtorAddr (ObjAddr, 0);	/* calling to destructor */
		}
	}
}

int
_CompareTypes ( char* TypeCode )
{
	int i;

	for (i=0; i<_ThrowTypesCnt; i++)  {
		if ( strcmp(TypeCode,_ThrowTypes[i].TypeCode) == 0 )  {
			_CurrentPart = _ThrowTypes[i].Value;
			return 1;
		}
	}

	return 0;
}

void
_NullThrow ( DF Dtor )
{
	Dtor = (DF) _ThrowBufferDtor;

	if ( Dtor != 0 )
		Dtor (_ThrowParam, 0);	// Destructor call

	_CurrentPart = 0;
	_ThrowTypesCnt = 0;
	_StillUncaught = false;
	_ThrowBufferDtor = NULL;
}

int
_PrepareException (void)
{
	if (_InsideUnexpected)
		_NullThrow (NULL);
	else if (uncaught_exception()) {
		terminate ();
		assert (0);
	};

	_StillUncaught = true;
	_ThrowTypesCnt = 0;
	return 0;
}

void
_ReThrow ()
{
	if (_ThrowTypesCnt && !_StillUncaught)
		_DestroyObjects ();
	else
		terminate ();
}

static bool
check_for_unexpected (char *str)
{
	while (*str != 0) {
		for (int i=0; i<_ThrowTypesCnt; i++) {

#ifdef DEBUG
			printf ("_CheckUnexpected(%s): %s \n",
				str, _ThrowTypes[i].TypeCode
			);
#endif
			if (strcmp(str, _ThrowTypes[i].TypeCode)==0)
				return false;
		};

		str += strlen (str)+1;
	};

	return true;
}

void
_CheckUnexpected (char *str)
{
	check_for_unexpected (str);

	if (check_for_unexpected(str)) {

		try {
			_InsideUnexpected = true;
				// unexpected () may throw an exception

			unexpected ();
			assert (0); // unexpected () shall not return 

		} catch (...) {

			_InsideUnexpected = false;

			if (check_for_unexpected(str)) {

				_NullThrow (NULL);

				try {
					throw bad_exception ();

				} catch (bad_exception &) {

					if (check_for_unexpected(str))
						terminate ();
					else
						throw;
				}
			} else
				_DestroyObjects ();
		};
	} else
		_DestroyObjects ();

}
