//----------------------------------------------------------------
//
// Copyright 1998-2010 (c) RC Module.
// NeuroChip NM6405.
//
// nc_int05.h
// Header file for low level library.
// Contain declaration of timer, comm. port and interrupt managing function
//
//----------------------------------------------------------------

#ifndef _NC_INT_H
#define _NC_INT_H

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

///////////////////////////////////
//  
///////////////////////////////////

//      :  
//       
//     

//       
void StartTimerRepeat_T0();
void StartTimerRepeat_T1();
//      
void StartTimerOnce_T0();
void StartTimerOnce_T1();
//   
void StopTimer_T0();
void StopTimer_T1();
//    
int SetTimerCounter_T0( int );
int SetTimerCounter_T1( int );
//    
int GetTimerCounter_T0();
int GetTimerCounter_T1();

//     1,    -
int clock();

///////////////////////////////////
//  
///////////////////////////////////


typedef void (HandlerType)(void);

enum InterKind
{
    I_KIND_NMI =0,      //   
    I_KIND_OF,          //  
    I_KIND_EI,          //   
    I_KIND_ST,          //   
    I_KIND_PE,          //       
                        //        ,    
                        //            
    I_KIND_ARMH_IN=16,  //     ARMU
    I_KIND_NMUH_IN,     //      NMU
    I_KIND_TIMER0,      //     0
    I_KIND_MS_ITMR,     //       ms-
    I_KIND_DSPPU_CH,    //     DSPPU
    I_KIND_AUX_ITMR,    //  .  .   . . 
    I_KIND_DMAC,        //     DMAC
    I_KIND_TIMER1,      //     1
    I_KIND_DSPPU_PRG,   //  .     DSPPU
    I_KIND_ARML_IN,     //     ARMU
    I_KIND_NMUL_IN,     //      NMU
    I_KIND_PU,          //      
    I_KIND_STRB_ITMR,   //  .  .     
    I_KIND_DSPPU_HRD,   //        DSPPU
    I_KIND_MAX = I_KIND_DSPPU_HRD
};

//   -
extern HandlerType DefaultIntHandler;

//    
HandlerType*       //   
SetInterrupt( InterKind k, HandlerType f );

//     
HandlerType* GetInterrupt( InterKind k );

//      
HandlerType* ClearInterrupt( InterKind k );

//      
HandlerType* FlushInterrupts( );

enum InterMaskPSWR
{
    I_MASK_OF = 0x00000100,      //  
    I_MASK_EI = 0x00000080,      //   
    I_MASK_INM =0x00000040,      //   
    I_MASK_ST = 0x00000020       //   
};

enum InterMaskPeriphery
{
    I_IMR_NONE     =0,  //   
    I_IMR_ARMH_IN  =1<<(I_KIND_ARMH_IN-16),
    I_IMR_NMUH_IN  =1<<(I_KIND_NMUH_IN-16),
    I_IMR_TIMER0   =1<<(I_KIND_TIMER0-16),
    I_IMR_MS_ITMR  =1<<(I_KIND_MS_ITMR-16),
    I_IMR_DSPPU_CH =1<<(I_KIND_DSPPU_CH-16),
    I_IMR_AUX_ITMR =1<<(I_KIND_AUX_ITMR-16),
    I_IMR_DMAC     =1<<(I_KIND_DMAC-16),
    I_IMR_TIMER1   =1<<(I_KIND_TIMER1-16),
    I_IMR_DSPPU_PRG=1<<(I_KIND_DSPPU_PRG-16),
    I_IMR_ARML_IN  =1<<(I_KIND_ARML_IN-16),
    I_IMR_NMUL_IN  =1<<(I_KIND_NMUL_IN-16),
    I_IMR_PU       =1<<(I_KIND_PU-16),
    I_IMR_STRB_ITMR=1<<(I_KIND_STRB_ITMR-16),
    I_IMR_DSPPU_HRD=1<<(I_KIND_DSPPU_HRD-16)
};


//    .
void EnableInterrupts ( InterMaskPSWR, InterMaskPeriphery imp=I_IMR_NONE );

//     (   ).
void DisableInterrupts ( InterMaskPSWR );

//      .
void DisableInterruptsPeriphery ( InterMaskPeriphery );

//     
unsigned int GetInterruptMaskIMR ( void );


///////////////////////////////////
//  
///////////////////////////////////


//     
unsigned int LibintGetPSWR();

//    NMSCR
enum NmscrMasks
{
    NMSCR_NMUH  = 1 <<0,    //    , 
    NMSCR_NMUL  = 1 <<1,    //     
    NMSCR_ARMUH = 1 <<2,    //    , 
    NMSCR_ARMUL = 1 <<3,    //    
    NMSCR_NUMBER= 1 <<4,    //    
    NMSCR_TD    = 0xf <<5   //    ( 6901   -!!! )
};

unsigned int LibintGetNMSCR();
void LibintSetNMSCR( unsigned int );

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // _NC_INT_H
